require "prefabutil"

local assets=
{
    Asset("ANIM", "anim/mushroom_light.zip"),
    Asset("ANIM", "anim/mushroom_light2.zip"),
    Asset("ATLAS", "images/inventoryimages/mushroom_light.xml"),
    Asset("ATLAS", "images/inventoryimages/mushroom_light2.xml"),
    Asset( "ATLAS", "minimap/mushroom_light.xml" ),
    Asset("SOUND", "sound/wilson.fsb"),
}

local prefabs =
{
    "collapse_small",
}

local function onopen(inst)
    --inst.components.container:Open()
end

local function onclose(inst)
    --inst.components.container:Close()
end

local function IsLightOn(inst)
    return inst.Light:IsEnabled()
end

local light_str =
{
    {radius = 2.5, falloff = .85, intensity = 0.75},
    {radius = 3.25, falloff = .85, intensity = 0.75},
    {radius = 4.25, falloff = .85, intensity = 0.75},
    {radius = 5.5, falloff = .85, intensity = 0.75},
}

local colour_tint = { 0.4, 0.3, 0.25, 0.2, 0.1 }
local mult_tint = { 0.7, 0.6, 0.55, 0.5, 0.45 }

local sounds_1 =
{
    toggle = "dontstarve/wilson/lantern_LP", "loop",
    craft = "dontstarve/wilson/lantern_off",
}
local sounds_2 =
{
    toggle = "dontstarve/wilson/lantern_LP", "loop",
    colour = "dontstarve/wilson/lantern_on",
    craft = "dontstarve/wilson/lantern_off",
}

local function ClearSoundQueue(inst)
    if inst._soundtask ~= nil then
    inst._soundtask:Cancel()
    inst._soundtask = nil
    end
end

local function OnQueuedSound(inst, soundname)
    inst._soundtask = nil
    inst.SoundEmitter:PlaySound(soundname)
end

local function QueueSound(inst, delay, soundname)
    if inst._soundtask ~= nil then
    inst._soundtask:Cancel()
    end
    inst._soundtask = inst:DoTaskInTime(delay, OnQueuedSound, soundname)
end

local function IsRedCap(item) return item.prefab == "red_cap" end
local function IsGreenCap(item) return item.prefab == "green_cap" end
local function IsBlueCap(item) return item.prefab == "blue_cap" end

local function UpdateLightState(inst,item)
    if inst:HasTag("burnt") then
    return
end
    ClearSoundQueue(inst)
    local sound = inst.onlywhite and sounds_1 or sounds_2
    local num_batteries = #inst.components.container:FindItems(function(item)
    return item:HasTag("lightbattery") or item:HasTag("spore") end)
    local was_on = IsLightOn(inst)

    if num_batteries > 0 then
    inst.Light:SetRadius(light_str[num_batteries].radius)
    inst.Light:SetFalloff(light_str[num_batteries].falloff)
    inst.Light:SetIntensity(light_str[num_batteries].intensity)

if not inst.onlywhite then

    local r = #inst.components.container:FindItems(IsRedCap)
    local g = #inst.components.container:FindItems(IsGreenCap)
    local b = #inst.components.container:FindItems(IsBlueCap)
    inst.Light:SetColour(colour_tint[g+b + 1] + r/6, colour_tint[r+b + 1] + g/6, colour_tint[r+g + 1] + b/6)
    inst.AnimState:SetMultColour(mult_tint[g+b + 1], mult_tint[r+b + 1], mult_tint[r+g + 1], 1)
end

    if not was_on then
    inst.Light:Enable(true)
    inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
end
    inst.AnimState:PlayAnimation("idle_on")
    if not was_on or inst.onlywhite then
    inst.AnimState:PlayAnimation("turn_on")
    inst.AnimState:PushAnimation("idle_on", false)
    inst.SoundEmitter:PlaySound(sound.toggle)
    else
    inst.AnimState:PlayAnimation("colour_change")
    inst.AnimState:PushAnimation("idle_on", false)
    inst.SoundEmitter:PlaySound(sound.toggle)
    QueueSound(inst, 13 * FRAMES, sound.colour)
end
    else
    inst.Light:Enable(false)
    inst.AnimState:ClearBloomEffectHandle()
    inst.AnimState:SetMultColour(.7, .7, .7, 1)
    inst.AnimState:PlayAnimation("idle")

    if was_on then
    inst.AnimState:PlayAnimation("turn_off")
    inst.AnimState:PushAnimation("idle", false)
    inst.SoundEmitter:PlaySound(sound.toggle)
        end
    end
end

local function onworkfinished(inst)
    inst.components.lootdropper:DropLoot()
    local fx = SpawnPrefab("collapse_small")
    fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
    inst:Remove()
end

local function onworked(inst, worker, workleft)
    if workleft > 0 and not inst:HasTag("burnt") 
    then
    ClearSoundQueue(inst)
    inst.AnimState:PlayAnimation(IsLightOn(inst) and "hit_on" or "hit")
    inst.AnimState:PushAnimation(IsLightOn(inst) and "idle_on" or "idle", false)
    inst.components.container:DropEverything()
    inst.components.container:Close()
    end
end

local function onbuilt(inst)
    ClearSoundQueue(inst)
    inst.AnimState:PlayAnimation("place")
    inst.AnimState:PushAnimation("idle", false)
    inst.SoundEmitter:PlaySound(inst.onlywhite and sounds_1.craft or sounds_2.craft)
end

local function getstatus(inst)
    return (inst:HasTag("burnt") and "BURNT")
    or (IsLightOn(inst) and "ON")
    or "OFF"
end

local function onsave(inst, data)
    if inst.components.burnable ~= nil and inst.components.burnable:IsBurning()
    or inst:HasTag("burnt") 
    then
    data.burnt = true
    end
end

local function onload(inst, data)
    if data ~= nil and data.burnt then
    inst.components.burnable.onburnt(inst)
    end
end

local slotpos = 
{   
    Vector3(0,64+32+8+4,0), 
    Vector3(0,32+4,0),
    Vector3(0,-(32+4),0), 
    Vector3(0,-(64+32+8+4),0)
}

local function itemtest(inst, item, slot)
    if inst.prefab == "mushroom_light" then
    return 
    item.prefab == "lightbulb" 
else
    return 
    item.prefab == "lightbulb" or
    item.prefab == "red_cap" or 
    item.prefab == "green_cap" or
    item.prefab == "blue_cap"
    end
end

local function MakeMushroomLight(name, onlywhite, physics_rad)
    local assets =
{
    Asset("ANIM", "anim/"..name..".zip"),
    Asset("ANIM", "anim/ui_cookpot_1x4.zip"),
    Asset("ATLAS", "images/inventoryimages/mushroom_light.xml"),
    Asset("ATLAS", "images/inventoryimages/mushroom_light2.xml"),
    Asset( "ATLAS", "minimap/mushroom_light.xml" ),
}

local function fn()
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddLight()

    MakeObstaclePhysics(inst, physics_rad)

    inst.AnimState:SetBank(name)
    inst.AnimState:SetBuild(name)
    inst.AnimState:PlayAnimation("idle")
    inst.AnimState:SetMultColour(.7, .7, .7, 1)

    inst.Light:SetColour(.65, .65, .5)
    inst.Light:Enable(false)

    local minimap = inst.entity:AddMiniMapEntity()
    minimap:SetIcon("mushroom_light.tex")

    inst:AddTag("fridge")

    inst.onlywhite = onlywhite

    MakeSmallBurnable(inst, nil, nil, true)
    MakeSmallPropagator(inst)
    MakeSnowCovered(inst)

    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(3)
    inst.components.workable:SetOnFinishCallback(onworkfinished)
    inst.components.workable:SetOnWorkCallback(onworked)

    inst:AddComponent("inspectable")
    inst.components.inspectable.getstatus = getstatus

    inst:AddComponent("lootdropper")

    inst:AddComponent("container")
    inst.components.container.itemtestfn = itemtest
    inst.components.container:SetNumSlots(4)
    inst.components.container.widgetslotpos = slotpos
    inst.components.container.widgetanimbank = "ui_cookpot_1x4"
    inst.components.container.widgetanimbuild = "ui_cookpot_1x4"
    inst.components.container.widgetpos = Vector3(120,0,0)
    inst.components.container.side_align_tip = 100
    inst.components.container.acceptsstacks = false

    inst:ListenForEvent("onbuilt", onbuilt)
    inst:ListenForEvent("itemget", UpdateLightState)
    inst:ListenForEvent("itemlose", UpdateLightState)
    inst:ListenForEvent("burntup", ClearSoundQueue)

    inst.components.container.onopenfn = onopen
    inst.components.container.onclosefn = onclose

    inst.OnSave = onsave
    inst.OnLoad = onload

    return inst
end

    return Prefab(name, fn, assets, prefabs)
end

return 
    MakeMushroomLight("mushroom_light", true, .25),
    MakeMushroomLight("mushroom_light2", false, .4),
    MakePlacer("mushroom_light_placer", "mushroom_light", "mushroom_light", "idle"),
    MakePlacer("mushroom_light2_placer", "mushroom_light2", "mushroom_light2", "idle")
